@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: qx_imager.cpp,v 1.12 2003/11/07 22:08:16 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: qx_imager.cpp,v 1.13 2005/11/17 20:09:08 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -427,7 +427,7 @@ QXImager::allocateCell(int r, int g, int b)
       }
 		  
       int closest_distance=0xffff;
-      XColor closest_cell;
+      XColor closest_cell = cmap_cells[0];
       for(int pix=0;pix<256;pix++)
       {
 	 XColor & c=cmap_cells[pix];
