@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: MapBorder.cpp,v 1.6 2003/11/07 22:08:17 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: MapBorder.cpp,v 1.7 2006/01/19 15:20:40 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -329,7 +329,7 @@ MapOval::ma_generatePieces(void)
    if (!oval->get_rmin()) return;
    
    int last_x=xc, last_y=yc;
-   int points=(int) (2*Pi*sqrt(oval->get_rmax()))+1;
+   int points=(int) (2*Pi*sqrt((double)(oval->get_rmax())))+1;
    TArray<int> xx(points-1), yy(points-1);
    int i, cnt=0;
    for(i=0;i<points;i++)
