@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: MapDraw.cpp,v 1.6 2003/11/07 22:08:17 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: MapDraw.cpp,v 1.7 2006/01/19 15:20:41 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -371,7 +371,7 @@ MapDraw::drawOval(GPixmap & pm, const GRect & grect, u_int32 color)
    pix.y=iy=(int) rint(fy);
    drawOvalPixel(pix);
 
-   a2=a/sqrt(2);
+   a2=a/sqrt(2.0);
 
       // Integrate ~1/8 of the oval (y>x>0)
    for(ix++;ix<a2;ix++)
