@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: qd_base.cpp,v 1.18 2003/11/07 22:08:17 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: qd_base.cpp,v 1.19 2005/12/24 12:45:01 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -287,7 +287,7 @@ QDBase::setDjVuImage(const GP<DjVuImage> & _dimg, int do_redraw)
    if (mode_src==SRC_ANT) 
      setMode(IDC_DISPLAY_COLOR, 0, SRC_ANT);
    if (rotate_src==SRC_ANT) 
-     setMode(IDC_ROTATE_0, 0, SRC_ANT);
+     setRotate(IDC_ROTATE_0, 0, SRC_ANT);
 
    decodeAnno(false);	// No redraw
    layout(0);		// Disallow redraw
