@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: qd_doc_info.cpp,v 1.11 2003/11/07 22:08:18 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: qd_doc_info.cpp,v 1.12 2005/12/28 07:54:50 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -553,7 +553,7 @@ QDDocInfo::slotShowInfo(void)
           const char * ptr;
           while((ptr=strstr(info, "icon\n")))
           {
-            GUTF8String tmp((const char *) info, ptr-info);
+            GUTF8String tmp((const char *)info, ptr-(const char*)info);
             tmp+="icon for page "+GUTF8String(start_page_num++);
             tmp+=ptr+4;
             info=tmp;
