@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: GMarginCache.h,v 1.7 2003/11/07 22:08:18 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: GMarginCache.h,v 1.8 2006/01/19 15:20:41 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifndef HDR_GMARGIN_CACHE
 #define HDR_GMARGIN_CACHE
@@ -307,7 +307,7 @@ GMarginCache<TPixmap>::nextRectToCache(int maxpixels, GRect & grect)
    
    if (outer_rect.isempty())
    {
-      int width=(int) sqrt(maxpixels*doc_width/doc_height);
+      int width=(int) sqrt((double)(maxpixels*doc_width/doc_height));
       if (!width) width++;
       int height=maxpixels/width;
       if (!height) height++;
