@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: DjVmDoc.cpp,v 1.9 2003/11/07 22:08:20 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: DjVmDoc.cpp,v 1.10 2005/05/25 20:24:52 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -62,6 +62,7 @@
 #endif
 
 #include "DjVmDoc.h"
+#include "DjVmNav.h"
 #include "DataPool.h"
 #include "IFFByteStream.h"
 #include "GOS.h"
@@ -230,6 +231,14 @@ DjVmDoc::delete_file(const GUTF8String &id)
    dir->delete_file(id);
 }
 
+void 
+DjVmDoc::set_djvm_nav(GP<DjVmNav> n)
+{
+  if (n && ! n->isValidBookmark())
+    G_THROW("Invalid bookmark data");
+  nav = n;
+}
+
 GP<DataPool>
 DjVmDoc::get_data(const GUTF8String &id) const
 {
@@ -272,7 +281,7 @@ get_name(const DjVmDir::File &file)
 
 void
 DjVmDoc::write(const GP<ByteStream> &gstr,
-  const GMap<GUTF8String,void *> &reserved)
+               const GMap<GUTF8String,void *> &reserved)
 {
   DEBUG_MSG("DjVmDoc::write(): Storing document into the byte stream.\n");
   DEBUG_MAKE_INDENT(3);
@@ -371,6 +380,12 @@ DjVmDoc::write(const GP<ByteStream> &gstr,
   tmp_iff.put_chunk("DIRM");
   dir->encode(tmp_iff.get_bytestream(),do_rename);
   tmp_iff.close_chunk();
+  if (nav)
+    {
+      tmp_iff.put_chunk("NAVM");
+      nav->encode(tmp_iff.get_bytestream());
+      tmp_iff.close_chunk();
+    }
   tmp_iff.close_chunk();
   int offset=tmp_iff.tell();
 
@@ -392,6 +407,12 @@ DjVmDoc::write(const GP<ByteStream> &gstr,
   iff.put_chunk("DIRM");
   dir->encode(iff.get_bytestream(),do_rename);
   iff.close_chunk();
+  if (nav)
+    {
+      iff.put_chunk("NAVM");
+      nav->encode(iff.get_bytestream());
+      iff.close_chunk();
+    }
 
   for(pos=files_list;pos;++pos)
   {
@@ -534,6 +555,12 @@ DjVmDoc::write_index(const GP<ByteStream> &str)
    iff.put_chunk("DIRM");
    dir->encode(iff.get_bytestream());
    iff.close_chunk();
+   if (nav)
+     {
+       iff.put_chunk("NAVM");
+       nav->encode(iff.get_bytestream());
+       iff.close_chunk();
+     }
    iff.close_chunk();
    iff.flush();
 }
