@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: DjVuDocEditor.cpp,v 1.11 2003/11/07 22:08:20 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: DjVuDocEditor.cpp,v 1.13 2005/05/25 20:24:52 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -70,6 +70,7 @@
 #include "GURL.h"
 #include "DjVuAnno.h"
 #include "GRect.h"
+#include "DjVmNav.h"
 
 #include "debug.h"
 
@@ -191,11 +192,7 @@ DjVuDocEditor::init(const GURL &url)
    {
          // Suxx. I need to convert it NOW.
          // We will unlink this file in the destructor
-#ifndef UNDER_CE
       tmp_doc_url=GURL::Filename::Native(tmpnam(0));
-#else
-      tmp_doc_url=GURL::Filename::UTF8("tempFileForDjVu");
-#endif
       const GP<ByteStream> gstr(ByteStream::create(tmp_doc_url, "wb"));
       tmp_doc->write(gstr, true);        // Force DJVM format
       gstr->flush();
@@ -1472,6 +1469,14 @@ DjVuDocEditor::create_shared_anno_file(void (* progress_cb)(float progress, void
    }
 }
 
+void 
+DjVuDocEditor::set_djvm_nav(GP<DjVmNav> n)
+{
+  if (n && ! n->isValidBookmark())
+    G_THROW("Invalid bookmark data");
+  djvm_nav = n;
+}
+
 GP<DjVuFile>
 DjVuDocEditor::get_shared_anno_file(void)
 {
