@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: DjVuMessageLite.cpp,v 1.12 2003/12/01 22:57:39 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: DjVuMessageLite.cpp,v 1.13 2005/04/27 16:34:13 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -126,8 +126,6 @@ static const char namestring[]="name";
 static const char valuestring[]="value";
 static const char numberstring[]="number";
 static const char bodystring[]="BODY";
-static const char headstring[]="HEAD";
-static const char includestring[]="INCLUDE";
 static const char messagestring[]="MESSAGE";
 
 GPList<ByteStream> &
