@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: GContainer.h,v 1.15 2004/05/13 15:16:34 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: GContainer.h,v 1.18 2006/03/14 19:50:27 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifndef _GCONTAINER_H_
 #define _GCONTAINER_H_
@@ -130,11 +130,87 @@ namespace DJVU {
     L\'eon Bottou <leonb@research.att.com> -- initial implementation.\\
     Andrei Erofeev <eaf@geocities.com> -- bug fixes.
     @version 
-    #$Id: GContainer.h,v 1.15 2004/05/13 15:16:34 leonb Exp $# */
+    #$Id: GContainer.h,v 1.18 2006/03/14 19:50:27 leonb Exp $# */
 //@{
 
 
 
+// ------------------------------------------------------------
+// HASH FUNCTIONS
+// ------------------------------------------------------------
+
+
+/** @name Hash functions
+    These functions let you use template class \Ref{GMap} with the
+    corresponding elementary types. The returned hash code may be reduced to
+    an arbitrary range by computing its remainder modulo the upper bound of
+    the range.
+    @memo Hash functions for elementary types. */
+//@{
+
+/** Hashing function (unsigned int). */
+static inline unsigned int 
+hash(const unsigned int & x) 
+{ 
+  return x; 
+}
+
+/** Hashing function (int). */
+static inline unsigned int 
+hash(const int & x) 
+{ 
+  return (unsigned int)x;
+}
+
+/** Hashing function (long). */
+static inline unsigned int
+hash(const long & x) 
+{ 
+  return (unsigned int)x;
+}
+
+/** Hashing function (unsigned long). */
+static inline unsigned int
+hash(const unsigned long & x) 
+{ 
+  return (unsigned int)x;
+}
+
+/** Hashing function (const void *). */
+static inline unsigned int 
+hash(const void * const & x) 
+{ 
+  return (unsigned long) x; 
+}
+
+/** Hashing function (float). */
+static inline unsigned int
+hash(const float & x) 
+{ 
+  // optimizer will get rid of unnecessary code  
+  unsigned int *addr = (unsigned int*)&x;
+  if (sizeof(float)<2*sizeof(unsigned int))
+    return addr[0];
+  else
+    return addr[0]^addr[1];
+}
+
+/** Hashing function (double). */
+static inline unsigned int
+hash(const double & x) 
+{ 
+  // optimizer will get rid of unnecessary code
+  unsigned int *addr = (unsigned int*)&x;
+  if (sizeof(double)<2*sizeof(unsigned int))
+    return addr[0];
+  else if (sizeof(double)<4*sizeof(unsigned int))
+    return addr[0]^addr[1];
+  else
+    return addr[0]^addr[1]^addr[2]^addr[3];    
+}
+
+
+
 // ------------------------------------------------------------
 // HELPER CLASSES
 // ------------------------------------------------------------
@@ -332,7 +408,6 @@ public:
 protected:
   const Traits &traits;
   void  *data;
-  GPBufferBase gdata;
   int   minlo;
   int   maxhi;
   int   lobound;
@@ -394,8 +469,6 @@ public:
       invocation of this conversion operator. */
   operator const TYPE* () const
     { return ((const TYPE*)data)-minlo; }
-  operator const TYPE* ()  // suppress warning with gcc-2.95
-    { return ((const TYPE*)data)-minlo; }
   // -- ALTERATION
   /** Erases the array contents. All elements in the array are destroyed.  
       The valid subscript range is set to the empty range. */
@@ -749,7 +822,7 @@ public:
   GListBase & operator= (const GListBase & gl);
   GPosition firstpos() const { return GPosition(head.next, (void*)this); }
   GPosition lastpos() const { return GPosition(head.prev, (void*)this); }
-  int isempty() const { return nelem==0; };
+  bool isempty() const { return nelem==0; };
   GPosition nth(unsigned int n) const;
   void empty();
 };
@@ -845,7 +918,7 @@ public:
   // -- TEST
   /** Tests whether a list is empty.  
       Returns a non zero value if the list contains no elements. */
-  int isempty() const 
+  bool isempty() const 
     { return this->nelem==0; }
   /** Compares two lists. Returns a non zero value if and only if both lists
       contain the same elements (as tested by #TYPE::operator==(const TYPE&)#
@@ -1158,7 +1231,7 @@ public:
     { return firstpos(); }    
   /** Tests whether the associative map is empty.  
       Returns a non zero value if and only if the map contains zero entries. */
-  int isempty() const
+  bool isempty() const
     { return this->nelems==0; }
   /** Searches an entry for key #key#.  If the map contains an entry whose key
       is equal to #key# according to #KTYPE::operator==(const KTYPE&)#, this
@@ -1266,87 +1339,6 @@ public:
 };
 
 
-// ------------------------------------------------------------
-// HASH FUNCTIONS
-// ------------------------------------------------------------
-
-
-/** @name Hash functions
-    These functions let you use template class \Ref{GMap} with the
-    corresponding elementary types. The returned hash code may be reduced to
-    an arbitrary range by computing its remainder modulo the upper bound of
-    the range.
-    @memo Hash functions for elementary types. */
-//@{
-
-/** Hashing function (unsigned int). */
-static inline unsigned int 
-hash(const unsigned int & x) 
-{ 
-  return x; 
-}
-
-/** Hashing function (int). */
-static inline unsigned int 
-hash(const int & x) 
-{ 
-  return (unsigned int)x;
-}
-
-/** Hashing function (long). */
-static inline unsigned int
-hash(const long & x) 
-{ 
-  return (unsigned int)x;
-}
-
-/** Hashing function (unsigned long). */
-static inline unsigned int
-hash(const unsigned long & x) 
-{ 
-  return (unsigned int)x;
-}
-
-/** Hashing function (void *). */
-static inline unsigned int 
-hash(void * const & x) 
-{ 
-  return (unsigned long) x; 
-}
-
-/** Hashing function (const void *). */
-static inline unsigned int 
-hash(const void * const & x) 
-{ 
-  return (unsigned long) x; 
-}
-
-/** Hashing function (float). */
-static inline unsigned int
-hash(const float & x) 
-{ 
-  // optimizer will get rid of unnecessary code  
-  unsigned int *addr = (unsigned int*)&x;
-  if (sizeof(float)<2*sizeof(unsigned int))
-    return addr[0];
-  else
-    return addr[0]^addr[1];
-}
-
-/** Hashing function (double). */
-static inline unsigned int
-hash(const double & x) 
-{ 
-  // optimizer will get rid of unnecessary code
-  unsigned int *addr = (unsigned int*)&x;
-  if (sizeof(double)<2*sizeof(unsigned int))
-    return addr[0];
-  else if (sizeof(double)<4*sizeof(unsigned int))
-    return addr[0]^addr[1];
-  else
-    return addr[0]^addr[1]^addr[2]^addr[3];    
-}
-
 
 //@}
 //@}
