@@ -51,13 +51,13 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: GException.h,v 1.9 2003/11/07 22:08:21 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: GException.h,v 1.11 2005/11/12 15:52:25 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifndef _GEXCEPTION_H_
 #define _GEXCEPTION_H_
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 #if NEED_GNUG_PRAGMAS
 # pragma interface
@@ -116,19 +116,44 @@
     L\'eon Bottou <leonb@research.att.com> -- initial implementation.\\
     Andrei Erofeev <eaf@geocities.com> -- fixed message memory allocation.
     @version 
-    #$Id: GException.h,v 1.9 2003/11/07 22:08:21 leonb Exp $# */
+    #$Id: GException.h,v 1.11 2005/11/12 15:52:25 leonb Exp $# */
 //@{
 
 #include "DjVuGlobal.h"
 
+// Check if compiler supports native exceptions
+#ifdef HAVE_CONFIG_H
+# if HAVE_EXCEPTIONS
+#  define CPP_SUPPORTS_EXCEPTIONS
+# endif
+#else
+# if defined(_MSC_VER)
+#  define CPP_SUPPORTS_EXCEPTIONS
+# endif
+# if defined(__MWERKS__)
+#  define CPP_SUPPORTS_EXCEPTIONS
+# endif
+# if defined(__EXCEPTIONS)
+#  define CPP_SUPPORTS_EXCEPTIONS
+# endif
+#endif
+// Decide which exception model to use
+#ifndef CPP_SUPPORTS_EXCEPTIONS
+# ifndef USE_EXCEPTION_EMULATION
+#  define USE_EXCEPTION_EMULATION
+# endif
+#endif
+
+#ifdef USE_EXCEPTION_EMULATION
+# include <setjmp.h>
+#endif
+
 #ifdef HAVE_NAMESPACES
 namespace DJVU {
 # ifdef NOT_DEFINED // Just to fool emacs c++ mode
 }
+# endif
 #endif
-#endif
-
-
 
 /** Exception class.  
     The library always uses macros #G_TRY#, #G_THROW#, #G_CATCH# and #G_ENDCATCH# for
@@ -226,24 +251,6 @@ private:
 #undef G_THROW_APPLICATION
 #undef G_THROW_OTHER
 
-// Check if compiler supports native exceptions
-#if defined(_MSC_VER)
-#define CPP_SUPPORTS_EXCEPTIONS
-#endif
-#if defined(__MWERKS__)
-#define CPP_SUPPORTS_EXCEPTIONS
-#endif
-#if defined(__EXCEPTIONS)
-#define CPP_SUPPORTS_EXCEPTIONS
-#endif
-// Decide which exception model to use
-#ifndef CPP_SUPPORTS_EXCEPTIONS
-#ifndef USE_EXCEPTION_EMULATION
-#define USE_EXCEPTION_EMULATION
-#endif
-#endif
-
-
 #ifndef USE_EXCEPTION_EMULATION
 
 // Compiler supports ANSI C++ exceptions.
@@ -278,8 +285,6 @@ public:
 // Compiler does not support ANSI C++ exceptions.
 // Emulate with setjmp/longjmp.
 
-#include <setjmp.h>
-
 class GExceptionHandler {
 public:
   jmp_buf jump;
@@ -310,8 +315,8 @@ public:
 #define G_EMTHROW(ex) GExceptionHandler::emthrow(ex)
 #else
 #define G_THROW_TYPE(m,xtype) GExceptionHandler::emthrow \
-  (GException(m, __FILE__, __LINE__,0, xtype)) no_return
-#define G_EMTHROW(ex) GExceptionHandler::emthrow(ex) no_return
+  (GException(m, __FILE__, __LINE__,0, xtype))
+#define G_EMTHROW(ex) GExceptionHandler::emthrow(ex)
 #endif
 
 #endif // !CPP_SUPPORTS_EXCEPTIONS
