@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: debug.h,v 1.11 2004/04/17 23:56:11 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: debug.h,v 1.12 2005/05/27 14:26:01 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifndef _DEBUG_H_
 #define _DEBUG_H_
@@ -126,7 +126,7 @@ namespace DJVU {
     @memo 
     Macros for printing debug messages.
     @version 
-    #$Id: debug.h,v 1.11 2004/04/17 23:56:11 leonb Exp $#
+    #$Id: debug.h,v 1.12 2005/05/27 14:26:01 leonb Exp $#
     @author
     Andrew Erofeev <eaf@geocities.com> -- initial implementation \\
     Leon Bottou <leonb@research.att.com> -- cleanups */
@@ -185,6 +185,8 @@ namespace DJVU {
 #  define _DEBUG
 # endif
 
+class GUTF8String;
+
 // ------------ SUPPORT
 
 class DjVuDebug // DJVU_CLASS
@@ -216,6 +218,7 @@ public:
   DjVuDebug &	operator<<(unsigned long i);
   DjVuDebug &	operator<<(const char * const ptr);
   DjVuDebug &	operator<<(const unsigned char * const ptr);
+  DjVuDebug&    operator<<(const GUTF8String &ptr);
   DjVuDebug &	operator<<(float f);
   DjVuDebug &	operator<<(double d);
   DjVuDebug &	operator<<(const void * const p);
