@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: csepdjvu.cpp,v 1.15 2005/07/23 15:15:19 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: csepdjvu.cpp,v 1.20 2006/02/21 17:27:34 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -134,7 +134,7 @@
     @author
     L\'eon Bottou <leonb@research.att.com>
     @version
-    #$Id: csepdjvu.cpp,v 1.15 2005/07/23 15:15:19 leonb Exp $# */
+    #$Id: csepdjvu.cpp,v 1.20 2006/02/21 17:27:34 leonb Exp $# */
 //@{
 //@}
 
@@ -332,7 +332,19 @@ BufferByteStream::read_pair(int &x, int &y)
 {
   int c;
   x = y = 0;
-  return read_integer(x) && expect(c, ":") && read_integer(y);
+  expect(c, "-");
+  if (! read_integer(x)) 
+    return false;
+  if (c == '-') 
+    x = -x;
+  if (! expect(c, ":"))
+    return false;
+  expect(c, "-");
+  if (! read_integer(y)) 
+    return false;
+  if (c == '-')
+    y = -y;
+  return true;
 }
 
 bool 
@@ -1167,7 +1179,7 @@ Comments::parse_comment_line(BufferByteStream &bs)
              bs.skip(" \t") && bs.read_pair(mark->dx,mark->dy) &&
              bs.skip(" \t") && bs.read_geometry(mark->r) &&
              bs.skip(" \t") && bs.read_ps_string(mark->s) ) )
-        G_THROW("msepdjvu: corrupted file (syntax error in text comment)");
+        G_THROW("csepdjvu: corrupted file (syntax error in text comment)");
       textmark(mark);
       return true;
     }
@@ -1228,7 +1240,7 @@ Comments::textmark(GP<TxtMark> mark)
       int shy = (mark->y - lasty) * 100 / fontsize;
       int inter = dirx * shx + diry * shy;
       if ( (dirx == lastdirx) && (diry == lastdiry) &&
-           (inter > -50) && (inter < 300) && 
+           (inter > -150) && (inter < 300) && 
            abs(diry * shx + dirx * shy) < 80 )
         mark->inter = inter;
       else
@@ -1259,7 +1271,8 @@ Comments::textflush(void)
       // compute word list
       GP<TxtMark> word;
       GPList<TxtMark> words;
-      for (GPosition p=lastline; p; ++p)
+	  { // extra nesting for windows
+        for (GPosition p=lastline; p; ++p)
         {
           TxtMark *mark = lastline[p];
           if (word && mark->inter > wordsep) 
@@ -1280,6 +1293,7 @@ Comments::textflush(void)
               word->r.recthull(word->r, mark->r);
             }
         }
+	  }
       if (word)
         {
           if (! allspaces(word->s))
