@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: djvuserve.cpp,v 1.21 2004/03/30 15:00:03 leonb Exp $
-// $Name: release_3_5_16 $
+// $Id: djvuserve.cpp,v 1.23 2006/02/21 17:27:34 leonb Exp $
+// $Name: release_3_5_17 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -384,7 +384,9 @@ main(int argc, char ** argv)
                   "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n"
                   "<HTML><HEAD><TITLE>400 Error</TITLE></HEAD><BODY>\n"
                   "<H1>%s</H1>The requested URL '%s' cannot be processed.<P>\n"
+#ifdef DJVULIBRE_VERSION
                   "<HR><ADDRESS>djvuserve/DjVuLibre-" DJVULIBRE_VERSION "</ADDRESS>\n"
+#endif
                   "</BODY></HTML>\n",
                   (const char *) cause,
                   (const char *) g().pathinfo );
@@ -396,5 +398,6 @@ main(int argc, char ** argv)
       exit(10);
     }
   G_ENDCATCH;
-  exit(0);
+  // return code zero
+  return 0;
 }
